##############################################
# R script for RSiena assignment on the
# co-evolution of alcohol consumption
# and friendship networks.
# version of 5 February 2017
# written by Christian Steglich
##############################################

# This R-script guides you through an RSiena
# analysis of alcohol consumption co-evolving
# with a dynamic, school cohort based
# friendship network in a Scottish school;
# three yearly waves of data were collected by
# Lynn Michell and Patrick West of the Medical
# Research Council / Medical Sociology Unit
# of the University of Glasgow in the school
# years 1994-95 until 1996-97.
# See also file "Lab-coevolution.pdf".

##############################################
# Contents:
#     (1) preparatory steps in R
#     (2) inspection of data
#     (3) preparing objects for RSiena
#     (4) model specification / selection
#         of effects
#     (5) model estimation
##############################################

# ==========================
# (1) preparatory steps in R
# ==========================

# if RSiena is not yet installed, un-hash & run the following
# line to install the RSiena package from R-Forge:
# install.packages("RSiena",repos="http://R-Forge.R-project.org") 

# load RSiena commands to the R namespace:
library(RSiena)

# check software version:
packageVersion("RSiena")
# should always be reasonably recent...

# set working directory to where you want to store output files:
setwd("<type here the path to the directory>")
# Windows users should note that Windows-specific backslashes "\"
# in a directory path need to be replaced by either forward slashes
# "/" or double backslashes "\\" to obtain an R-compatible path!
list.files()

# ======================
# (2) inspection of data
# ======================

# The s50 data are part of the RSiena package. For information
# about this data set, visit 
# http://www.stats.ox.ac.uk/~snijders/siena/s50_data.htm

# alcohol use:
dim(s50a) # 50 students, 3 time points
s50a[1:5,]
apply(s50a,FUN=table,MARGIN=2,useNA='always')

# network change between subsequent observations:
table(s501,s502,useNA='always')
table(s502,s503,useNA='always')
# & total:
table(s501,s503,useNA='always')

# alcohol change between subsequent observations:
table(s50a[,1],s50a[,2],useNA='always')
table(s50a[,2],s50a[,3],useNA='always')
# & total:
table(s50a[,1],s50a[,3],useNA='always')

# ================================
# (3) preparing objects for RSiena
# ================================

# identify dependent network variable:
friendship <- sienaDependent(array(c(s501,s502,s503),
	dim=c(dim(s501),3)))

# identify dependent behavior variable:
drinking <- sienaDependent(s50a,type="behavior")

# bind data together for Siena analysis:
CoEvolutionData <- sienaDataCreate(friendship,drinking)
CoEvolutionData

# create a model specification object for the data:
CoEvolutionModel <- getEffects(CoEvolutionData)
CoEvolutionModel

# if desired, write descriptive summary to protocol file (optional):
print01Report(CoEvolutionData,modelname='CoEvol-init')
# older software versions require CoEvolutionModel as 2nd argument

# ================================================
# (4) model specification / selection of effects
# ================================================
  
# inspect portfolio of possible effects:
effectsDocumentation(CoEvolutionModel)
# note how in column "name" we now have two names, indicating
# the network part and the behaviour part of the model

# specify the model according to the assignment, noting that the
# following choices are not the only good ones for most hypotheses:

# Hypothesis a); expected sign: +
CoEvolutionModel <- includeEffects(CoEvolutionModel,isolate,
	interaction1="friendship",name="drinking")
# here, instead of 'isolate', e.g., also 'indegree' could have
# been chosen, with expected sign -

# Hypothesis b); expected sign: +
CoEvolutionModel <- includeEffects(CoEvolutionModel,egoX,
	interaction1="drinking",name="friendship")

# Hypothesis c); expected sign: +
CoEvolutionModel <- includeEffects(CoEvolutionModel,simX,
	interaction1="drinking",name="friendship")
# here, instead of 'simX' also 'egoXaltX' could have been
# chosen as operationalisation

# Hypothesis d); expected sign: -
CoEvolutionModel <- includeEffects(CoEvolutionModel,altX,
	interaction1="drinking",name="friendship")

# Hypothesis e); expected sign: +
CoEvolutionModel <- includeEffects(CoEvolutionModel,avSim,
	interaction1="friendship",name="drinking")
# here, instead of 'avSim' also 'totSim', 'avAlt', or 'totAlt'
# could have been chosen

# add structural effects as control variables:
CoEvolutionModel <- includeEffects(CoEvolutionModel,recip,transTrip,
	transRecTrip,cycle3,	name="friendship")

# full model specification looks like this now:
CoEvolutionModel

# ====================
# (5) model estimation
# ====================

# create an object with options for algorithm tuning:
CoEvolutionOptions <- sienaAlgorithmCreate(useStdInits=FALSE,
	projname='CoEvol-results',MaxDegree=c(friendship=6),seed=786840)

# estimate the model, using two processor cores and suppressing the
# graphical user interface:
CoEvolutionResults <- siena07(CoEvolutionOptions,data=CoEvolutionData,
	effects=CoEvolutionModel,batch=TRUE,verbose=TRUE,
	useCluster=TRUE,nbrNodes=2)
CoEvolutionResults # converged

# We can now proceed to interpretation and conclusions regarding
# the hypotheses in the assignment: only statement (c) is supported,
# marginally supportive evidence for statement (e)...

# to give approximate p-values: 
parameter <- CoEvolutionResults$effects$effectName
estimate <- CoEvolutionResults$theta
st.error <- sqrt(diag(CoEvolutionResults$covtheta))
normal.variate <- estimate/st.error
p.value.2sided <- 2*pnorm(abs(normal.variate),lower.tail=FALSE)
data.frame(parameter,
	estimate=round(estimate,3),
	st.error=round(st.error,3),
	normal.variate=round(normal.variate,2),
	p.value=round(p.value.2sided,4)
)

# This script can be amended by two auxiliary analyses: 
# >>A<< moderation analysis (interaction terms with reciprocity)
# >>B<< goodness-of-fit inspection
# For this purpose, we save the intermediate results:
save.image('CoEvolutionResults.RData')
