##############################################
# R script for illustrating the multiple
# imputation of missing behaviour data for
# analysis with RSiena.
# version of 5 February 2017
# written by Christian Steglich
##############################################

# load RSiena commands:
library(RSiena)
packageVersion("RSiena")

# set working directory to where the data are:
setwd("<put your directory name here>")
list.files()

# read the datasets:
load("Glasgow-friendship.RData")
load("Glasgow-substances.RData")
load("Glasgow-demographic.RData")
ls()

# recode valued friendship data: 
friendship.1[friendship.1==2] <- 1
friendship.2[friendship.2==2] <- 1
friendship.3[friendship.3==2] <- 1

# identify structural absence:
absence <- data.frame(
	T1=colSums(friendship.1,na.rm=TRUE)==10*nrow(friendship.1),
	T2=colSums(friendship.2,na.rm=TRUE)==10*nrow(friendship.2),
	T3=colSums(friendship.3,na.rm=TRUE)==10*nrow(friendship.3)
)
colSums(absence)

# check percentage of alcohol-missings for structurally present actors:
prop.table(table(alcohol[!absence$T1,1],useNA='always'))
prop.table(table(alcohol[!absence$T2,2],useNA='always'))
prop.table(table(alcohol[!absence$T3,3],useNA='always'))
# 12%, 18% and 7%, respectively

# For illustration, we focus on the period T1->T2 which has a
# more severe (bot not critical) missingness problem.

# First let us run an analysis using RSiena's default imputation
# (LOCF, reduced impact on estimates - see manual section 4.3.2).

# define data:
thedata <- sienaDataCreate(
	friendship = sienaDependent(
		array(c(friendship.1,friendship.2),dim=c(160,160,2))
	),
	alcohol = sienaDependent(alcohol[,1:2],type='behavior'),
	sexF = coCovar(sex.F)
)
thedata

# get effects table and specify model:
theeffects <- getEffects(thedata)
effectsDocumentation(theeffects)
theeffects <- includeEffects(theeffects,transTrip,transRecTrip)
	name="friendship")
theeffects <- includeEffects(theeffects,sameX,
	name="friendship",interaction1="sexF")
theeffects <- includeEffects(theeffects,egoX,altX,simX,
	name="friendship",interaction1="alcohol")
theeffects <- includeEffects(theeffects,indeg,outdeg,avSim,
	name="alcohol",interaction1="friendship")
theeffects <- includeEffects(theeffects,effFrom,
	name="alcohol",interaction1="sexF")
theeffects

# create estimation options object:
theoptions <- sienaAlgorithmCreate(useStdInits=FALSE,
	projname='theresults',MaxDegree=c(friendship=6),seed=123)

# estimate model (took almost 5 minutes on my machine - skip in class!):
theresults <- siena07(theoptions,data=thedata,effects=theeffects,
	batch=FALSE,verbose=FALSE,useCluster=TRUE,initC=TRUE,nbrNodes=2)
theresults

# store results for possible later use:
#save(theresults,file="theresults.RData")
load("theresults.RData")

# Now let us proceed to multiple imputation with the R-package "mice":
#install.packages("mice") # if needed
library(mice)
?mice

# prepare data frame for imputation:
alcohol.plus.predictors <- data.frame(
	alcohol,age,sex.F,tobacco,cannabis,1*absence
)
# include indegrees and outdegrees, too:
alcohol.plus.predictors$od1 <- rowSums(friendship.1==1)
alcohol.plus.predictors$od2 <- rowSums(friendship.2==1)
alcohol.plus.predictors$od3 <- rowSums(friendship.3==1)
alcohol.plus.predictors$id1 <- colSums(friendship.1==1,na.rm=TRUE)
alcohol.plus.predictors$id2 <- colSums(friendship.2==1,na.rm=TRUE)
alcohol.plus.predictors$id3 <- colSums(friendship.3==1,na.rm=TRUE)
alcohol.plus.predictors$id1[absence$T1] <- NA
alcohol.plus.predictors$id2[absence$T2] <- NA
alcohol.plus.predictors$id3[absence$T3] <- NA
# strange coding above is to avoid counting the value 10 as a number

# chack data frame:
head(alcohol.plus.predictors)

# run a default imputation and generate five imputed data sets:
imputed <- mice(alcohol.plus.predictors,m=5,seed=123)
imputed
# for what the default is, see "?mice": it is predictive mean matching.

# check out what the five imputed alcohol values for t1 and t2 are:
imputed$imp$t1
imputed$imp$t2
# these can now be used to augment the alcohol data!

# make five augmented alcohol data sets:
# first copy the alcohol information five times...
alc.imputed <- rep(list(alcohol[,1:2]),5)
# ...then impute:
for (imp in 1:5) {
	alc.imputed[[imp]][is.na(alcohol[,1]),1] <- imputed$imp$t1[,imp]
	alc.imputed[[imp]][is.na(alcohol[,2]),2] <- imputed$imp$t2[,imp]
}
alc.imputed

# now continue with RSiena...

# make five augmented RSiena data sets:
data.imputed <- list()
for (imp in 1:5) {
	data.imputed[[imp]] <- sienaDataCreate(
		friendship = sienaDependent(
			array(c(friendship.1,friendship.2),dim=c(160,160,2))
		),
		alcohol = sienaDependent(alc.imputed[[imp]],type='behavior'),
		sexF = coCovar(sex.F)
	)
}
data.imputed

# estimate all of them:
# WARNING: takes 5 times 5 minutes on my machine! Skip in class!
results.imputed <- list()
for (imp in 5:5) {
	(results.imputed[[imp]] <- siena07(theoptions,
		data=data.imputed[[imp]],effects=theeffects,
		batch=FALSE,verbose=FALSE,useCluster=TRUE,initC=TRUE,nbrNodes=2
	))
}
results.imputed

# store results for possible later use:
#save(results.imputed,file="results-imputed.RData")
load("results-imputed.RData")


# Next, we need to pool the results according to the procedures implemented
# in the mice-package.
?pool
?as.mira
# For this, we need a list of parameter estimates called "coef" and a list
# of variance-covariance matrix estimates of the estimators at the estimate
# called "vcov". This info needs to be extracted from the RSiena results,
# where it is stored as "theta" and "covtheta", respectively.

# define functions for harvesting relevant info from the results:
coef.sienaFit <- function(x) {return(x$theta)}
vcov.sienaFit <- function(x) {return(x$covtheta)}

# harvest relevant info from the results:
harvested <- as.mira(results.imputed)

# pool results:
pooled.results <- data.frame(
	effect=theeffects$effectName[theeffects$include],
	round(summary(pool(harvested)),4)
) # hmm, there is a warning, let us ignore it for this course..
pooled.results

# save for later, if needed:
save.image("results imputation lab.RData")

# CONCLUSIONS: for the alcohol-related effects, standard errors
# become smaller when working with multiple imputation compared
# to RSiena's built-in imputation procedure. For other
# parameters, there seems to be no difference.

# LITERATURE:
# ^^^^^^^^^^^
# Huisman, M., & Steglich, C. (2008). Treatment of non-response in 
#	longitudinal network studies. Social Networks, 30(4), 297-308.
# Rubin, D.B. (1987). Multiple Imputation for Nonresponse in Surveys.
#	New York: John Wiley and Sons.
# van Buuren, S., and Groothuis-Oudshoorn, K. (2011). mice: Multivariate 
#	Imputation by Chained Equations in R. Journal of Statistical 
#	Software, 45(3), 1-67. http://www.jstatsoft.org/v45/i03/ 
